@@ -81,7 +81,7 @@ void R_nc4_inq_varid_hier( int *ncid, char **varname, int *returned_grpid, int *
 void R_nc4_get_vara_text( int *ncid, int *varid, int *start, int *count, char **tempstore, char **data, int *retval );
 void R_nc4_put_vara_text( int *ncid, int *varid, int *start, int *count, char **data, int *retval );
 
-void R_nc4_enddef( int *ncid );
+void R_nc4_enddef( int *ncid, int *retval );
 void R_nc4_sync  ( int *ncid );
 void R_nc4_close ( int *ncid );
 
@@ -108,6 +108,8 @@ SEXP R_nc4_get_vara_numvarid( SEXP sx_nc, SEXP sx_varid, SEXP sx_start, SEXP sx_
 SEXP R_ncu4_getListElement(SEXP list, char *str);
 SEXP R_nc4_get_vara_string( SEXP sx_nc, SEXP sx_varid, SEXP sx_start, SEXP sx_count );
 
+SEXP R_nc4_inq_libvers( void );
+
 /* For C calls that don't use SEXP type args */
 static const
 R_CMethodDef cMethods[] = {
@@ -168,7 +170,7 @@ R_CMethodDef cMethods[] = {
 	{"R_nc4_get_vara_text", 	(DL_FUNC) &R_nc4_get_vara_text,  	7},
 	{"R_nc4_put_vara_text", 	(DL_FUNC) &R_nc4_put_vara_text,  	6},
 
-	{"R_nc4_enddef", 		(DL_FUNC) &R_nc4_enddef,  		1},
+	{"R_nc4_enddef", 		(DL_FUNC) &R_nc4_enddef,  		2},
 	{"R_nc4_sync", 			(DL_FUNC) &R_nc4_sync,  		1},
 	{"R_nc4_close", 		(DL_FUNC) &R_nc4_close,  		1},
 
@@ -199,6 +201,8 @@ R_CallMethodDef callMethods[] = {
 	{"R_ncu4_getListElement", 	(DL_FUNC) &R_ncu4_getListElement, 	2},
 	{"R_nc4_get_vara_string", 	(DL_FUNC) &R_nc4_get_vara_string, 	4},
 
+	{"R_nc4_inq_libvers", 		(DL_FUNC) &R_nc4_inq_libvers,  		0},
+
 	NULL
 };
 
@@ -424,6 +428,11 @@ SEXP Rsx_nc4_get_vara_double( SEXP sx_ncid, SEXP sx_varid, SEXP sx_start, SEXP s
 	imvstate	= INTEGER(sx_imvstate)[0];
 	missval		= REAL   (sx_missval )[0];
 
+	/*
+	Rprintf( "Rsx_nc4_get_vara_double: entering with ncid=%d varid=%d, fixmiss=%d, imvstate=%d, missval=%lf\n",
+		ncid, varid, fixmiss, imvstate, missval );
+	*/
+
 	/* Get number of dimensions in this variable */
 	err = nc_inq_varndims( ncid, varid, &ndims );
 	if( err != NC_NOERR ) {
@@ -1012,8 +1021,10 @@ void R_nc4_put_att_text( int *ncid, int *varid, char **attname,
 	/* For some reason the C interface does not include the nc_type for this call */
 
 	attlen = strlen(attribute[0]);
+
 	*retval = nc_put_att_text(*ncid, *varid, attname[0], 
 		attlen, attribute[0] );
+
 	if( *retval != NC_NOERR ) 
 		Rprintf( "Error in R_nc4_put_att_text: %s\n", 
 			nc_strerror(*retval) );
@@ -2016,13 +2027,15 @@ void R_nc4_inq_varid_hier( int *ncid, char **varname, int *returned_grpid, int *
 }
 
 /*********************************************************************/
-void R_nc4_enddef( int *ncid )
+void R_nc4_enddef( int *ncid, int *retval )
 {
 	int	err;
 	err = nc_enddef(*ncid);
 	if( err != NC_NOERR ) 
 		Rprintf( "Error in R_nc4_enddef: %s\n", 
 			nc_strerror(err) );
+
+	*retval = err;
 }
 
 /*********************************************************************/
@@ -2538,6 +2551,34 @@ Rprintf( "warning, no match found for element %s\n", str );
 	return elmt;
 }
 
+/*********************************************************************************/
+SEXP R_nc4_inq_libvers( void ) 
+{
+	char 	buf[1000];
+	char	*tstr;
+	int	nn, kk;
+	SEXP	sx_retval, sx_string;
+
+	strncpy( buf, nc_inq_libvers(), 999 );
+
+	nn = strlen(buf);
+
+	tstr = R_alloc( nn+1, sizeof(char) );
+	for( kk=0; kk<nn; kk++ ) {
+		tstr[kk] = buf[kk];
+		}
+	tstr[nn] = '\0';
+
+	sx_string = PROTECT( mkChar( tstr ));
+
+	sx_retval = PROTECT( NEW_CHARACTER( 1 ));
+	SET_STRING_ELT( sx_retval, 0, sx_string );
+
+	UNPROTECT( 2 );
+	
+	return( sx_retval );
+}
+
 /*********************************************************************************
  * Read vlen strings given the numeric varid, start, and count to use
  */
